@@ -142,7 +142,7 @@ BEGIN {
 	   command =>		[undef, 	'read/write'],
 	   parameters =>	[undef,		'read/write'],
 	   PARENT		=>	[undef,		'read/write'],
-	   running		=>	["yes",		'read/write'],
+	   running		=>	["yes",	'read/write'],
 	   readfh		=>	[undef, 	'read/write'],
 	   writefh		=>	[undef, 	'read/write'],
 
@@ -172,6 +172,13 @@ BEGIN {
 	    }
 	    return keys %list;
 	}
+	
+	my $id = 0;
+	sub nextID{  # gives a unique output filename for plugins under Windows
+		return $id++
+	}
+	
+	
 	} #end of BEGIN block
 }
 
@@ -211,62 +218,114 @@ sub run {
 	my $commandline = "$command $args";
 	$commandline =~ /(\s*)(.*)/;   # remove any leading spaces
 	$commandline = $2;
-	return if (!$commandline);
+	return unless $commandline;
 	#print "\t*** executing $PLUGINS_DIR/$commandline\n";
-	my ($rdr, $wtr) = (FileHandle->new, FileHandle->new);
+	
+	if ($^O =~ /win32/i){
+	#if (1){
+		$self->runWindows($commandline, $PLUGINS_DIR);
+	} else {
+		my ($rdr, $wtr) = (FileHandle->new, FileHandle->new);
+			
+		$self->readfh($rdr);
+		$self->writefh($wtr);
 		
-	$self->readfh($rdr);
-	$self->writefh($wtr);
-	
-	my $pid = open2($rdr, $wtr, "perl $PLUGINS_DIR$commandline");
+		my $pid = open2($rdr, $wtr, "perl $PLUGINS_DIR$commandline");
 
-	return 0  unless $pid;	
-	# needto make the filehandles non-blocking	
+		return 0  unless $pid;	
+		# needto make the filehandles non-blocking	
 
-	my $flags = '';
-	
-	unless ($^O =~ /Win32/){
-		fcntl($rdr, F_GETFL, $flags) or die "can't get flags for $rdr:  $!\n";
-		$flags |= O_NONBLOCK;
-		fcntl($rdr, F_SETFL, $flags) or die "can't set the flags for $rdr: $!\n";
-		# now do for the write filehandle
-		fcntl($wtr, F_GETFL, $flags) or die "can't get flags for $wtr:  $!\n";
-		$flags |= O_NONBLOCK;
-		fcntl($wtr, F_SETFL, $flags) or die "can't set the flags for $wtr: $!\n";
-	}
-	
-	my $message;
-	$rdr->autoflush;
-	$wtr->autoflush;
-
-	while ($self->running eq "yes"){
-		my $receive = <$rdr>;
-		chomp $receive;
-		unless (length($receive)){
-			$PARENT->update;
-			next;
+		my $flags = '';
+		
+		unless ($^O =~ /Win32/){
+			fcntl($rdr, F_GETFL, $flags) or die "can't get flags for $rdr:  $!\n";
+			$flags |= O_NONBLOCK;
+			fcntl($rdr, F_SETFL, $flags) or die "can't set the flags for $rdr: $!\n";
+			# now do for the write filehandle
+			fcntl($wtr, F_GETFL, $flags) or die "can't get flags for $wtr:  $!\n";
+			$flags |= O_NONBLOCK;
+			fcntl($wtr, F_SETFL, $flags) or die "can't set the flags for $wtr: $!\n";
 		}
-		if ($receive =~ /<KILLED>/){print "\n\nplugin KILLED\n\n"; undef $self; return}
-		if ($receive eq "<EXIT>"){print "EXITING plugin\n"; undef $self; return}
-		if ($receive =~ /<\/REQUEST>/){  # look for end-of-request element
-			$message .= $receive;			# if found then the request is complete; 
-			my $return = $self->parseMessage($message); # parse it, and send it to the approprioate genqurie subroutine
-			#print "sending $wtr $return\n"; # print what was sent to the swcreen
-			my $done = $self->respondToPlugin($wtr, $return);  # send the result in small packages to be read
-			#print "returned $done\n"; # print what was sent to the swcreen
-			$message = "";
-			$PARENT->update;
-			next;
 		
-		} else {
-			#print "\n\n\nintermediate $receive\n\n\n";
-			$message .= $receive;		# if message is still incomplete then append current string
-			$PARENT->update;
-			next;
+		my $message;
+		$rdr->autoflush;
+		$wtr->autoflush;
+
+		while ($self->running eq "yes"){
+			my $receive = <$rdr>;
+			chomp $receive;
+			unless (length($receive)){
+				$PARENT->update;
+				next;
+			}
+			if ($receive =~ /<KILLED>/){print "\n\nplugin KILLED\n\n"; undef $self; return}
+			if ($receive eq "<EXIT>"){print "EXITING plugin\n"; undef $self; return}
+			if ($receive =~ /<\/REQUEST>/){  # look for end-of-request element
+				$message .= $receive;			# if found then the request is complete; 
+				my $return = $self->parseMessage($message); # parse it, and send it to the approprioate genqurie subroutine
+				#print "sending $wtr $return\n"; # print what was sent to the swcreen
+				my $done = $self->respondToPlugin($wtr, $return);  # send the result in small packages to be read
+				#print "returned $done\n"; # print what was sent to the swcreen
+				$message = "";
+				$PARENT->update;
+				next;
+			
+			} else {
+				#print "\n\n\nintermediate $receive\n\n\n";
+				$message .= $receive;		# if message is still incomplete then append current string
+				$PARENT->update;
+				next;
+			}
 		}
 	}
 }
 
+sub runWindows {
+	my ($self, $commandline, $PLUGINS_DIR) = @_;
+	$|=1;
+	
+	my ($crap1, $crap2) = (FileHandle->new,FileHandle->new);
+	
+	open2 ($crap1, $crap2, "perl $PLUGINS_DIR$commandline") || die "failed to execute";
+	#my $id = $self->nextID;
+	if (-e "REQUEST"){unlink "REQUEST"};
+	if (-e "REQUESTtemp"){unlink "REQUESTtemp"};
+	if (-e "RESPONSEtemp"){unlink "RESPONSEtemp"};
+	if (-e "RESPONSE"){unlink "RESPONSE"};
+				
+	while ($self->running eq "yes"){
+		if (-e "REQUEST"){  # if the output file exists
+			#print "file found\n\a";
+			if (-r "REQUEST"){
+				#print "readable\n\a";
+				open MESSAGE,"<REQUEST";
+				my $receive = join "", <MESSAGE>;
+				close MESSAGE;
+				#print "Message : $receive\n";
+				unlink "REQUEST" || die "cant delete REQUEST $!";
+				while (-e "REQUEST"){next};  # let the system catch up
+				if ($receive =~ /<KILLED>/){print "\n\nplugin KILLED\n\n"; undef $self; return}
+				if ($receive eq "<EXIT>"){print "EXITING plugin\n"; undef $self; return}
+				if ($receive =~ /<\/REQUEST>/){  # look for end-of-request element
+					my $return = $self->parseMessage($receive); # parse it, and send it to the approprioate genqurie subroutine
+					#print "sending $return\n"; # print what was sent to the swcreen
+					my $done = $self->respondToWindowsPlugin($return);  # send the result in small packages to be read
+					#print "returned $done\n"; # print what was sent to the swcreen
+					$self->PARENT->update;
+					next;
+				} else {
+					print "uninterpretable message...\n";
+					unlink "REQUEST" || die "cant delete request $!";
+					while (-e "REQUEST"){next};  
+					return;
+				}
+			}
+		}
+		$self->PARENT->update;
+	}
+}
+
+
 sub Quit {
 	my ($self) = @_;
 	my $fh = $self->writefh;
@@ -289,6 +348,21 @@ sub respondToPlugin {
 	return $return;
 }	
 
+sub respondToWindowsPlugin {
+	my ($self, $return) = @_;
+	#print "RESPONSING!!\n";
+	#open CHECK, ">CHECK";
+	#close CHECK;
+	
+	open RESPONSE, ">RESPONSEtemp" || die "failed to open response $!";;
+	print RESPONSE $return;
+	close RESPONSE;
+	
+	rename "RESPONSEtemp", "RESPONSE"; # Win32 cant lock a file, so write all output, then rename it
+	
+	return $return;
+}	
+
 sub parseMessage {
 	my ($self, $message)=@_;
 	my $return;
@@ -308,8 +382,9 @@ sub parseMessage {
 	elsif ($req_code eq "getActiveSequences"){$return = $self->getActiveSequences($message)}
 	elsif ($req_code eq "selectFeatures"){$return = $self->selectFeatures($message)}
 	elsif ($req_code eq "mapFeature"){$return = $self->mapFeature($message)}
-	else {print "\n\ncant understand code $req_code \n\tfrom message $message\n\n";$self->defaultDisplay($message)
-		  }
+	else {
+		print "\n\ncant understand code $req_code \n\tfrom message $message\n\n";$self->defaultDisplay($message)
+	}
 	$return = "<GQ_RESPONSE>\n<$req_code>\n$return\n</$req_code>\n</GQ_RESPONSE>\n";
 	return $return;
 
@@ -739,17 +814,23 @@ sub mapFeature {
 	unless (defined $frame){$frame = "."}
 	
 	
-	# print "$contig && $start $stop && $feature && $source && $strand && $frame && $score\n";
+	#print "PLUGIN HANDLER RECEIVED $contig && $start $stop && $feature && $source && $strand && $frame && $score\n";
 	return &FAILED("Not properly formatted Genquire GFF") unless (defined($contig) && defined($start) && defined($stop) && defined($feature) && defined($source) && defined($strand) && defined($frame) && defined($score));
+	#print "CONTINUING...\n\n";
 	
 	my $MAP;	
 	foreach my $Map(@{$self->PARENT->MapWindows}){
 		my ($SeqCanvas, $stt, $stp, $ctg, $MapWindow) = @{$Map}; # only SeqCanvas is interesting here...
 		my $ctgid = $SeqCanvas->MapSeq->primary_id; # based on MapSeq->primary_id (contig::start-length)
-		if (($ctgid == $contig)){ # find the appropriate open map if there is one, 
+		#print "comparing *$ctgid* to *$contig* with $SeqCanvas\n";
+		#print length $ctgid, " length for ctgid\n";
+		#print length $contig, " length for contig\n";
+		if ($ctgid =~/$contig/ || $contig =~ /$ctgid/){ # find the appropriate open map if there is one,
+			#print "GRABBED!\n";
 			$MAP = $SeqCanvas;  # grab it
 		}
 	}
+	#print "Found Map $MAP is its name\n";
 	my $feat; my $newcontig; my $is_map = 0; # is_contig is a boolean saying whether this is a full contig or an active map
 	# they may a "pure" contig analysis or an open MapSeq contig analysis
 	# These can be distingished by the name of the contig...
@@ -761,19 +842,27 @@ sub mapFeature {
 		$start = $cstart + $start - 1;  # and add them -1 off-by-one be damned!
 		$stop = $cstart + $stop - 1;  # same with stop
 		$is_map = 1;  # flag it as a MapType
+	}  else {
+		$newcontig = $contig;
 	}
 	
+	#print "newcontig $newcontig\n";
 	if ($import) {
 		if ($is_map  && !$MAP){return}  # if this doesn't appear to be an open map, but the name has that form, then presume that the client has buggered up.
-		# but, we *can* import features into the database that are not masquerading as MapSequences
-		# since tese might represent legitimate contigs.
+		# but, we CAN import features into the database that are not masquerading as open-map features
+		# since these might represent contigs in our database that are just not displayed at the moment
+		#print "IMPORTING ismap $is_map  MAP $MAP\n";
 		my $gff_string = "$newcontig\t$source\t$feature\t$start\t$stop\t$score\t$strand\t$frame\t$msg";
+		#print "GFF to Genquire: $gff_string\n\n";
 		$feat = $self->PARENT->load_gff_string($gff_string);  # this creates a Genquire feature object, and loads it into the database
-		$feat->par_pos($MAP->MapSeq->bc_start+$MAP->MapSeq->start-1);
-		unless ($feat){return &FAILED("Failed to import feature into the database")}
+		#print "returned from feature creation $feat\n";		
+		#print "returned from par_pos\n";		
+		unless ($feat){print "FAILED!!!\a";return &FAILED("Failed to import feature into the database")}
+		#print "succeeded $feat\n";
 		unless ($is_map && $MAP) {return 1}  # check again if it is a map open to draw on, and if so, then go ahead
+		$feat->par_pos($MAP->MapSeq->bc_start+$MAP->MapSeq->start-1);
 		$MAP->mapFeatures("draft", [$feat]);  # and map the feature
-		return;
+		return 1;
 	} else {
 		# if they dont want to import, then we only care about it having map open
 		unless ($MAP) {return}
