@@ -325,27 +325,39 @@ sub selectFreeSequence {
 	my ($start, $end, $strand);
 	
 	if ($MapObj->{-orientation} eq "horizontal"){
-		my $x1 = $MapObj->DraftCanvas->canvasx($MapObj->dragx1);
-		my $x2 = $MapObj->DraftCanvas->canvasx($MapObj->dragx2);
-		if ($x1 > $x2){($x1, $x2) = ($x2, $x1)};
-		my $length = (($x2 - $x1)/($MapObj->DraftMap->{scale_factor}));   #/
 		
-		my ($left, $right) = $MapObj->DraftCanvas->xview;
-		$start = ($x1/($MapObj->DraftMap->{scale_factor})) - ($left * ($MapObj->MapSeq->length)); #/
-		$end = $start + $length;
+		my ($offset_left, $junk1, $offset_right, $junk3) = @{$MapObj->DraftCanvas->cget('-scrollregion')};
+		
+		my $x_event1 = $MapObj->dragx1;  # where does the canvas see the mouse-down event
+		my $x_event2 = $MapObj->dragx2;  # where does the canvas see the mouse-up event
+		
+		my $x1 = ($x_event1-$offset_left)/($offset_right-$offset_left);  # given the zoom, what proportion of the canvas do these events intersect?
+		my $x2 = ($x_event2-$offset_left)/($offset_right-$offset_left);
+		if ($x1 > $x2){($x1, $x2) = ($x2, $x1)};
+		
+		$start = int($x1 * $MapObj->MapSeq->length); # start is proportion time length, same for stop
+		$end = int($x2 * $MapObj->MapSeq->length);
+		
 		$start=0 if ($start<0)
 	} else {
-		my $x1 = $MapObj->DraftCanvas->canvasx($MapObj->dragy1);
-		my $x2 = $MapObj->DraftCanvas->canvasx($MapObj->dragy2);
-		if ($x1 > $x2){($x1, $x2) = ($x2, $x1)};
-		my $length = (($x2 - $x1)/($MapObj->DraftMap->{scale_factor}));   #/
 		
-		my ($left, $right) = $MapObj->DraftCanvas->yview;
-		$start = ($x1/($MapObj->DraftMap->{scale_factor})) - ($left * ($MapObj->MapSeq->length)); #/
-		$end = $start + $length;
+		my ($junk1, $offset_top, $junk2, $offset_bottom) = @{$MapObj->DraftCanvas->cget('-scrollregion')};
+		
+		my $y_event1 = $MapObj->dragy1;
+		my $y_event2 = $MapObj->dragy2;
+
+		my $y1 = ($y_event1-$offset_top)/($offset_bottom-$offset_top);  # given the zoom, what proportion of the canvas do these events intersect?
+		my $y2 = ($y_event2-$offset_top)/($offset_bottom-$offset_top);
+		
+		if ($y1 > $y2){($y1, $y2) = ($y2, $y1)};
+		
+		$start = int($y1 * $MapObj->MapSeq->length); #/
+		$end = int($y2 * $MapObj->MapSeq->length);
 		$start=0 if ($start<0)
 	}
 
+	#print "start $start  stop $end\n\n\n";
+	
 	while (!$strand || !($strand =~/^[+-]$/)){
 		$strand = "+";
 		my $AskFeature = $self->DialogBox(-title => 'Forward or Reverse Strand?',
@@ -850,12 +862,7 @@ sub new {
     my $rbtnOrientationV;
 	$self->{orientation} = $self->ori || "horizontal";
     $rbtnOrientationH = $q->Radiobutton(
-		-command => sub {
-			unless ($self->ori){
-				$self->ori("horizontal")
-			} else {
-				$self->{orientation} = $self->ori
-			}		
+		-command => sub {	
 		},
 		-background => $bg,
 		-width => 15,
@@ -865,12 +872,7 @@ sub new {
 		-variable => \$self->{orientation});
 	
     $rbtnOrientationV = $q->Radiobutton(
-		-command => sub {
-			unless ($self->ori){
-				$self->ori("vertical")
-			} else {
-				$self->{orientation} = $self->ori
-			}		
+		-command => sub {	
 		},
 		-background => $bg,
 		-width => 15,
@@ -1033,13 +1035,7 @@ sub openMainWindows {
 	my ($self) = @_;
 	
 	return if ($self->{contig} eq "" or ($self->{stop} < $self->{start}) or (!($self->{orientation} =~ /horizontal|vertical/)));  # make sure the values are valid
-				
-	unless ($self->ori){
-		$self->ori($self->{orientation})
-	} else {
-		$self->{orientation} = $self->ori
-	}		
-					
+								
 	# if a light-blue details text-box doesn't already exist then create one
 	unless ($self->TextOutput) {
     	my $topText = $self->Toplevel(-title => "Details of Selected Items");
@@ -1458,38 +1454,53 @@ sub mapEST{
 
 
 sub parse_gff_string {
+	# most of this code is nicked directly from the GFF.pm module
+	#  _from_gff2_string, which I wrote several months ago
+	
     my ($self,$string)=@_;
-
-    my ($seq, $source, $primary, $start, $end, $score, $strand, $frame, $attribs) = split(/\t+/, $string);
-
-    ####### parse comments ###### Mark's code except for %taghash
+	
+	my ($seq, $source, $primary, $start, $end, $score, $strand, $frame, @attribs) = split(/\t+/, $string);
     my %taghash;
-    $attribs =~ s/\#(.*)$//;				  # remove comments field (format:  #blah blah blah...  at the end of the GFF line)
 
-    my @key_vals = split /;/, $attribs; # attributes are semicolon-delimited
+	my $attribs = join '', @attribs;  # just in case the rule against tab characters has been broken
 
-    foreach my $pair ( @key_vals ) {
-	my ($key, $values) = split /^\s*([\w\d]+)\s/, $pair; # separate the key from the value based on the = sign
-	my @values;
+	$attribs =~ s/\#(.*)$//;				  # remove comments field (format:  #blah blah blah...  at the end of the GFF line)
 
-	while ($values =~ s/"(.*?)"//) { # free text is quoted, so match each free-text block
-	    if ($1) {
-		push @values, $1;
-	    }
-	    # and push it on to the list of values (tags may have more than one value...)
+	my $attribs = join '', @attribs;  # just in case the rule against tab characters has been broken
+
+	my @att = split //, $attribs;         # split into individual characters
+	my $num = $#att;                 # count them
+	my $flag = 0;
+	my @parsed;		# this is needed to hold the characters that have been parsed	
+	for (my $a = 0; $a <= $num ; $a +=1){   # run through each character one at a time and check it
+		if ($att[$a] eq "\""){$flag=($flag==0)?1:0}  # flag up on entering quoted text, down on leaving it
+		if (($att[$a] eq ";") && $flag){$att[$a] = "INSERT_SEMICOLON_HERE"}  # replace semicolon with an unusual message if the quoted text flag is up
+		if (($att[$a] eq "#") && !$flag){last}  # an unquoted hash symbol means the beginning of the comments field - discard
+		push @parsed, $att[$a]                  # take the parsed character and push it onto the parsed list
 	}
 
-	my @othervals = split /\s+/, $values; # and what is left over should be space-separated non-free-text values
-	foreach my $othervalue (@othervals) {
-	    if (length($othervalue) > 0) {
-		push @values, $othervalue;
-	    }
-	    # get rid of any empty strings which might result from the split
+	my $attribs = join "", @parsed; # rejoin into a single string
+
+	my @key_vals = split /;/, $attribs;   # attributes are semicolon-delimited
+
+	foreach my $pair ( @key_vals ) {
+	   $pair =~ s/INSERT_SEMICOLON_HERE/;/g;  # replace semicolons that were removed from free-text above.
+	   my ($blank, $key, $values) = split  /^\s*([\w\d]+)\s/, $pair;	# separate the key from the value
+
+	   my @values;								
+
+	   while ($values =~ s/"(.*?)"//){          # free text is quoted, so match each free-text block and remove it from the $values string
+			push @values, $1;          # and push it on to the list of values (tags may have more than one value... and the value may be undef)
+	   }
+
+	   my @othervals = split /\s+/, $values;  # and what is left over should be space-separated non-free-text values
+	   foreach my $othervalue(@othervals){
+			if (CORE::length($othervalue) > 0){push @values, $othervalue}  # get rid of any empty strings which might result from the split
+	   }
+
+		$taghash{$key} = \@values;
+
 	}
-
-	$taghash{$key} = \@values;
-
-    }
     return ($seq, $source, $primary, $start, $end, $score, $strand, $frame,%taghash);
 }
 
